/*
 *
 * This file is a part of the CaosDB Project.
 *
 * Copyright (C) 2021 Timm Fitschen <t.fitschen@indiscale.com>
 * Copyright (C) 2021 IndiScale GmbH <info@indiscale.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see <https://www.gnu.org/licenses/>.
 *
 */

// A simple caosdb client
#include <iostream>
#include <memory>
#include <string>
#include "caosdb/constants.h"
#include "caosdb/connection.h"
#include "caosdb/authentication.h"
#include "caosdb/utils.h"
#include "caosdb/info.h"
#include "caosdb/entity.h"      // for Entity, EntityID
#include "caosdb/transaction.h" // for Transaction, UniqueResult

auto main() -> int {

  std::cout << "CaosDB (libcaosdb " << caosdb::LIBCAOSDB_VERSION_MINOR << "."
            << caosdb::LIBCAOSDB_VERSION_MINOR << "."
            << caosdb::LIBCAOSDB_VERSION_PATCH << ")\n"
            << "We don't miss the H of caos.\n"
            << std::endl;

  const auto pem_file =
    caosdb::utils::get_env_var("CAOSDB_SERVER_CERT", std::string());
  const auto host =
    caosdb::utils::get_env_var("CAOSDB_SERVER_HOST", "localhost");
  const auto port_str =
    caosdb::utils::get_env_var("CAOSDB_SERVER_GRPC_PORT_HTTPS", "8443");
  const auto port = std::stoi(port_str);
  const auto user = caosdb::utils::get_env_var("CAOSDB_USER", "admin");
  const auto password = caosdb::utils::get_env_var("CAOSDB_PASSWORD", "caosdb");

  // setup the connection
  auto auth =
    std::make_shared<caosdb::authentication::PlainPasswordAuthenticator>(
      user, password);
  auto cacert =
    std::make_shared<caosdb::connection::PemFileCACertProvider>(pem_file);
  auto config = std::make_shared<caosdb::connection::SslCaosDBConnectionConfig>(
    host, port, cacert, auth);
  caosdb::connection::CaosDBConnection connection(config);

  // get version info of the server
  const auto &v_info = connection.GetVersionInfo();
  std::cout << "Server Version: " << v_info->GetMajor() << "."
            << v_info->GetMinor() << "." << v_info->GetPatch() << "-"
            << v_info->GetPreRelease() << "-" << v_info->GetBuild()
            << std::endl;

  // retrieve an entity
  auto transaction(connection.CreateTransaction());
  transaction->RetrieveById("20");
  transaction->Execute();
  auto result_set(std::static_pointer_cast<caosdb::transaction::UniqueResult>(
    transaction->GetResultSet()));

  // print description
  std::cout << "Entity Description: "
            << result_set->GetEntity().GetDescription() << std::endl;

  return 0;
}
